// global localStorage,define
define(["build/models/course"], function(Course) {
    var db = {}

    db.loadCourses = function() {
        // check for old localstorage data and transfer over if needed
        if (localStorage.getItem("courses")) {
            db.transferOldCoursesDB()
        }
        var courseBase = localStorage.getItem("courseBase");
        // var courses = testInitializeStorage();
        var courses = courseBase ? JSON.parse(courseBase) : initializeStorage();
        return courses;
    }

    db.resetAllData = function() {
        initializeStorage()
    }

    db.transferOldCoursesDB = function() {
        var new_courses = []
        var courses = JSON.parse(localStorage.getItem('courses'))
        if (courses) {
            var n = 0
            for (key in courses) {
                var c = courses[key]
                var course_id = pad(n, 5)
                n++

                var new_tasks = []
                for (var i = 0; i < c.tasks.length; i++) {
                    c.tasks[i].course = course_id
                    c.tasks[i].id = pad(n, 5)
                    new_tasks.push(c.tasks[i])
                    n++
                }

                new_courses.push(new Course({
                    "code": key,
                    "id": course_id,
                    "name": c.name,
                    "tasks": new_tasks
                }))
            }
        }
        localStorage.setItem('courseBase', JSON.stringify(new_courses))
        localStorage.removeItem('courses')
    }

    db.getAllTasks = function() {
        var courseBase = localStorage.getItem("courseBase");
        var courses = courseBase ? JSON.parse(courseBase) : initializeStorage();
        var tasks = [];
        for (var i = courses.length - 1; i >= 0; i--) {
            tasks = tasks.concat(courses[i].tasks)
        };
        return tasks
    }

    db.getAllCourses = function() {
        var courseBase = localStorage.getItem("courseBase");
        var courses = courseBase ? JSON.parse(courseBase) : [];
        return courses;
    }

    db.deleteItem = function(id) {
        var courses = JSON.parse(localStorage.getItem("courseBase"));
        for (var i = 0; i < courses.length; i++) {
            if (courses[i].id === id) {
                courses.splice(i, 1);
            }
        }
        localStorage.setItem("courseBase", JSON.stringify(courses));
    }

    function pad(str, max) {
        str = str.toString();
        return str.length < max ? pad("0" + str, max) : str;
    }

    function testInitializeStorage() {
        var courses = [new Course({
            "code": "PHL411",
            "id": "00000",
            "name": "Philosophy",
            "tasks": [{
                "id": "00001",
                "name": "Essay 1",
                "course": "00000",
                "date": "2014-02-10",
                "weight": "20",
                "completed": "true",
                "grade": "89",
                "notes": ""
            }, {
                "course": "00000",
                "id": "00002",
                "name": "Essay 2",
                "date": "2014-02-14",
                "weight": "20",
                "completed": "true",
                "grade": "86",
                "notes": ""
            }, {
                "id": "00003",
                "name": "Essay 3",
                "course": "00000",
                "date": "2014-02-16",
                "weight": "20",
                "completed": "true",
                "grade": "75",
                "notes": ""
            }, {
                "id": "00004",
                "name": "Essay 4",
                "course": "00000",
                "date": "2014-04-10",
                "weight": "20",
                "completed": "false",
                "grade": "",
                "notes": ""
            }, {
                "id": "00005",
                "name": "Participation",
                "course": "00000",
                "date": "",
                "weight": "20",
                "completed": "false",
                "grade": "",
                "notes": ""
            }]
        }), new Course({
            "code": "CSC309",
            "id": "00006",
            "name": "Computer Science",
            "tasks": [{
                "id": "00007",
                "name": "A1",
                "course": "00006",
                "date": "2014-05-23",
                "weight": "20",
                "completed": "false",
                "grade": "",
                "notes": ""
            }, {
                "id": "00008",
                "name": "A2",
                "course": "00006",
                "date": "2014-02-15",
                "weight": "25",
                "completed": "false",
                "grade": "",
                "notes": ""
            }, {
                "id": "00009",
                "name": "A3",
                "course": "00006",
                "date": "2014-07-01",
                "weight": "20",
                "completed": "false",
                "grade": "",
                "notes": ""
            }, {
                "id": "00010",
                "name": "Participation",
                "course": "00006",
                "date": "",
                "weight": "10",
                "completed": "false",
                "grade": "",
                "notes": ""
            }, {
                "id": "00011",
                "name": "Final Exam",
                "course": "00006",
                "date": "",
                "weight": "25",
                "completed": "false",
                "grade": "",
                "notes": ""
            }]
        }), new Course({
            "code": "HPS318",
            "id": "00012",
            "name": "Canadian History",
            "tasks": [{
                "id": "00013",
                "name": "i1",
                "course": "00012",
                "date": "2014-02-12",
                "weight": "5",
                "completed": "true",
                "grade": "",
                "notes": ""
            }, {
                "id": "00014",
                "name": "i2",
                "course": "00012",
                "date": "2014-02-22",
                "weight": "10",
                "completed": "false",
                "grade": "",
                "notes": ""
            }, {
                "id": "00015",
                "name": "G1",
                "course": "00012",
                "date": "2014-02-30",
                "weight": "15",
                "completed": "false",
                "grade": "",
                "notes": ""
            }, {
                "id": "00016",
                "name": "i3",
                "course": "00012",
                "date": "2014-06-05",
                "weight": "10",
                "completed": "false",
                "grade": "",
                "notes": ""
            }, {
                "id": "00017",
                "name": "i4",
                "course": "00012",
                "date": "2014-06-23",
                "weight": "10",
                "completed": "false",
                "grade": "",
                "notes": ""
            }, {
                "course": "00012",
                "id": "00018",
                "name": "G2",
                "date": "2014-06-25",
                "weight": "15",
                "completed": "false",
                "grade": "",
                "notes": ""
            }, {
                "id": "00019",
                "name": "i5",
                "course": "00012",
                "date": "2014-07-05",
                "weight": "10",
                "completed": "false",
                "grade": "",
                "notes": ""
            }, {
                "course": "00012",
                "id": "00020",
                "name": "G3",
                "date": "2014-07-08",
                "weight": "25",
                "completed": "false",
                "grade": "",
                "notes": ""
            }]
        }), new Course({
            "code": "CRM101",
            "id": "001055",
            "name": "Intro to Criminology",
            "tasks": []
        })];
        localStorage.setItem("courseBase", JSON.stringify(courses));
        return JSON.parse(localStorage.getItem("courseBase"));
    }

    function initializeStorage() {
        var courses = []
        localStorage.setItem("courseBase", JSON.stringify(courses));
        return JSON.parse(localStorage.getItem("courseBase"));
    }

    return db;
});