/** @jsx React.DOM */
define([
'app', 
'build/controllers/course', 
'build/controllers/task'
], function(app, CourseController, TaskController) {

	var $ = Framework7.$;
	
	var CourseNavbar = React.createClass({
		render: function() {
			return (
				<div className="navbar-wrapper">
			        <div className="left">
			            <a href="#" className="back link">
			                <i className="icon icon-back"></i>
			                <span>Back</span>
			            </a>
			        </div>
			        <div className="center sliding">{this.props.model.code}</div>
			        <div className="right">
			            <a href={'task.html?cid=' + this.props.model.id} className="link icon-only open-panel"><i className="icon icon-plus">+</i></a>
			        </div>
			    </div>
			);
		}
	});

	var CoursePage = React.createClass({
		render: function() {
			return (
				<div className="page-content">
					<CourseSummary model = {this.props.model} />
					<MultiTasksList model = {this.props.model} />
				</div>
			)
		}
	})

	var CourseSummary = React.createClass({
		render: function() {
			var summary = this.get_summary();
			return (
				<div className="content-block course-summary">
					{/*<div className="content-block-title">{this.props.model.name}</div>*/}
					<div className="row course-name">
	                    <h1>{this.props.model.name}</h1>
	                </div>

	                <div className="row">
	                	<div className="col-33 cumulative-grade">
	                		<div className="row">{summary.cumulative + '%'}</div>
	                		<div className="row">Cumulative</div>
	                	</div>
	                	<div className="col-34 current-grade">{summary.current + '%'}</div>
	                	<div className="col-33 potential-grade">
	                		<div className="row">{summary.potential + '%'}</div>
	                		<div className="row">Potential</div>
	                	</div>
					</div>
	            </div>
			)
		},
		get_summary: function() {
		    var summary = {};
		    var sumWeights = 0; //for completed tasks
		    var cumulativeMark = 0
		    var totalWeight = 0; //all tasks
		    for (t in this.props.model.tasks) {
		        var task = this.props.model.tasks[t];
		        if (task.grade && task.weight) {
		            sumWeights = sumWeights + parseFloat(task.weight);
		            cumulativeMark = cumulativeMark + parseFloat(task.grade) * parseFloat(task.weight) / 100;
		        }
		        totalWeight = totalWeight + parseFloat(task.weight);
		    }
		    if (sumWeights == 0) {
		        summary.current = 0;
		    } else {
		        summary.current = (cumulativeMark / sumWeights * 100).toFixed(2);
		    }
		    summary.cumulative = cumulativeMark.toFixed(2);
		    summary.potential = (cumulativeMark + 100 - sumWeights).toFixed(2);
		    summary.totalWeight = totalWeight;
		    return summary;
		}
	})

	var MultiTasksList = React.createClass({
		render: function() {
			var tasks = this.categorize_tasks(this)
			return (
				<div>
					<div className="content-block course-tab-buttons">
		                <div className="buttons-row">
		                    <a href="#tab1" className="tab-link active button"><i className="fa fa-square-o"></i> Tasks</a>
		                    <a href="#tab2" className="tab-link button"><i className="fa fa-check-square-o"></i> Done</a>
		                    <a href="#tab3" className="tab-link button"><i className="fa fa-pencil-square-o"></i> Marked</a>
		                </div>
	                </div>
					<div className="tabs-animated-wrap">
		                <div className="tabs">
		                    <div id="tab1" className="tab active">
		                        <TaskList tasks={tasks.todo} />
		                    </div>

		                    <div id="tab2" className="tab">
		                        <TaskList tasks={tasks.done} />
		                    </div>

		                    <div id="tab3" className="tab">
		                        <TaskList tasks={tasks.marked} />
		                    </div>
		                </div>
		            </div>
	            </div>
			)
		},
		categorize_tasks: function(tasks) {
			var todo = [], marked = [], done = [];
			for (var i = this.props.model.tasks.length - 1; i >= 0; i--) {
				var t = this.props.model.tasks[i];
				if (t.grade || !isNaN(parseInt(t.grade))) {
					marked.push(t)
				} else if (t.completed == true) {
					done.push(t)
				} else {
					todo.push(t)
				}
			};
			return {
				todo: todo,
				marked: marked,
				done: done
			}
		}
	})

	var TaskList = React.createClass({
		render: function() {
			var sorted_tasks = _.sortBy(this.props.tasks, function(t) { return new Date(t.date) })
			var taskItems = [];
			for (var i = 0; i <= sorted_tasks.length - 1; i++) {
				taskItems.push(
					<TaskItem task={sorted_tasks[i]} key={sorted_tasks[i].id}/>
				)
			};
			return (
				<div className="list-block media-list">
                    <ul>
                    	{taskItems}
                    </ul>
                </div>
			)
		}
	})

	var SwipeoutComplete = React.createClass({
		render: function() {
			return (
				<div className="swipeout-actions-left">
                    <a href="#" className="swipeout-overswipe bg-green complete" onClick={this.handleComplete}>Complete</a>
                </div>
			)
		},
		handleComplete: function() {
			var id = this.props.task.id
			TaskController.init({id:id})
	    	app.f7.swipeoutComplete('.swipeout[id="' + id + '"]', function(){
	    		TaskController.completeTask()
	    	})
		}
	})

	var TaskItem = React.createClass({
		render: function() {
			var id = this.props.task.id
			var href = "task.html?id=" + id

			var swipe_left;
			if (this.props.task.completed != true) {
				swipe_left = <SwipeoutComplete task={this.props.task} />
			}

			var after = this.props.task.grade ? this.props.task.grade + '%' : this.props.task.date;

			var text = this.props.task.notes ? <div className="item-text">{this.props.task.notes}</div> : '';

			var subtitle = this.props.task.weight ? 'w: ' + this.props.task.weight + '%' : '';
			return (
				<li className="swipeout" id={id}>
                    <div className="swipeout-content">
                        <a href={href} className="item-content item-link">
                            <div className="item-inner">
                                <div className="item-title-row">
                                    <div className="item-title">{this.props.task.name}</div>
                                    <div className="item-after">{after}</div>
                                </div>
                                <div className="item-subtitle">{subtitle}</div>
                                {text}
                            </div>
                        </a>
                    </div>
                    {swipe_left}
                    <div className="swipeout-actions-right">
                        <a href="#" className="swipeout-overswipe bg-red delete" onClick={this.handleDelete}>Delete</a>
                    </div>
                </li>
			)
		},
		handleDelete: function() {
			var id = this.props.task.id
			app.f7.confirm('Are you sure?', 'Delete Task',
			    function() {
			    	TaskController.init({id:id})
			    	app.f7.swipeoutDelete('.swipeout[id="' + id + '"]', function(){
			    		TaskController.deleteTask()
			    	})
			    }, function() {
			    	app.f7.swipeoutClose('.swipeout-opened')
			    }
			)
		}
	})

	coursePage = null;

	function update_tabs_height() {
		var $ = Framework7.$;
		$('#course-page .tabs-animated-wrap').css('height', window.innerHeight - $('.tabs-animated-wrap').offset().top + 'px') // fill bottom part of window
	}

	return {
		init: function(query) {
			CourseController.init(query);
			courseNavbar = React.renderComponent(<CourseNavbar model = {CourseController.model}/>, document.getElementById('course-navbar'));
			coursePage = React.renderComponent(<CoursePage model = {CourseController.model} />, document.getElementById('course-page'));
			update_tabs_height()
		},
		receiveMessage: function(message) {
			switch (message) {
				case 'update':
					coursePage.setProps({model: CourseController.model});
					courseNavbar.setProps({model: CourseController.model});
					break;
			}
		}
	};
})